/*
 *  config.c
 *  mod_musicindex
 *
 *  $Id: config.c,v 1.41 2004/03/11 19:32:48 boudinr Exp $
 *
 *  Created by Thibaut VARENE on Thu Mar 20 2003.
 *  Copyright (c) 2003-2004 Regis BOUDIN
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1, or (at your option)
 *  any later version.
 *
 */

/**
 * @file 
 * Configuration handling and default settings.
 *
 * @author Regis Boudin
 * @version $Revision: 1.41 $
 * @date 2003-2004
 *
 * That file takes care of the module configuration, either via setting some
 * known default values, and/or by getting them from environmental configuration
 * variables passed to apache in its config file.
 *
 * @todo Regis, please document.
 */

#include "config.h"
#include "inf.h"

const char default_sort[] = "APNBTLRFMU";	/**< Default sort order */
const char default_fields[] = "TBLR";		/**< Default fields displayed */

/**
 * Generates a per-directory configuration.
 *
 * This function creates a new configuration structure and fills it with
 * default values.
 * 
 * @param p Apache pool for memory allocation.
 * @param dummy String given by apache we don't use.
 *
 * @return The newly created configuration structure.
 */
void *create_musicindex_config(apr_pool_t *p, char *dummy)
{
	unsigned short i;
	mu_config *new = (mu_config *)ap_pcalloc(p, sizeof(mu_config));

	for (i = 0; i < ARG_NUMBER; i++) {
		new->order[i] = SB_DEFAULT;
		new->fields[i] = '\0';
	}

	strcpy(new->fields, default_fields);
	strcpy(new->order, default_sort);

	new->title = ap_pstrdup(p, "Music");
	new->favicon = ap_pstrdup(p, "/musicindex/sound.png");
	new->cd_icon = ap_pstrdup(p, "/musicindex/general.png");
	new->small_cd_icon = ap_pstrdup(p, "/musicindex/directory.png");
	new->sound_icon = ap_pstrdup(p, "/musicindex/sound.png");
	new->fetch_icon = ap_pstrdup(p, "/musicindex/fetch.png");
	new->arrow = ap_pstrdup(p, "/musicindex/right_arrow.gif");
	new->css = ap_pstrdup(p, "/musicindex/musicindex.css");
	new->search = NULL;
	new->cache_path = NULL;
	new->iceserver = NULL;

	new->play_recursive = ~MI_RECURSIVE;
	new->options = MI_RECURSIVE;
	
	set_fctptrs(new);

	return (void *)new;
}

/**
 * Merge per-directory configurations.
 *
 * This function is supposed to merge two per-directory configurations. We
 * simply copy it from the parent directory.
 * 
 * @param p Apache pool for memory allocation.
 * @param basev Pointer to configuration structure.
 * @param addv Pointer to parent configuration structure.
 *
 * @return The newly created configuration structure.
 */
void *merge_musicindex_configs(apr_pool_t *p, void *basev, void *addv)
{
	mu_config *new = (mu_config *)ap_pcalloc(p, sizeof(mu_config));
	mu_config *add = (mu_config *)addv;

	strcpy(new->order, add->order);
	strcpy(new->fields, add->fields);

	new->title = ap_pstrdup(p, add->title);
	new->favicon = ap_pstrdup(p, add->favicon);
	new->cd_icon = ap_pstrdup(p, add->cd_icon);
	new->small_cd_icon = ap_pstrdup(p, add->small_cd_icon);
	new->sound_icon = ap_pstrdup(p, add->sound_icon);
	new->fetch_icon = ap_pstrdup(p, add->fetch_icon);
	new->arrow = ap_pstrdup(p, add->arrow);
	new->css = ap_pstrdup(p, add->css);
	new->search = ap_pstrdup(p, add->search);
	new->cache_path = ap_pstrdup(p, add->cache_path);
	new->iceserver = ap_pstrdup(p, add->iceserver);
					
	new->play_recursive = add->play_recursive;
	new->options = add->options;
	
	set_fctptrs(new);

	return new;
}

void sort_or_fields(cmd_parms *cmd, char *list, const char *optstr)
{
	char *r;
	unsigned short i = 0;

	while (optstr[0] && (i < ARG_NUMBER)) {
		r = ap_getword_conf(cmd->pool, &optstr);
		if (!strcasecmp(r, "track"))
			list[i++] = SB_TRACK;
		else if (!strcasecmp(r, "disc"))
			list[i++] = SB_POSN;
		else if (!strcasecmp(r, "length"))
			list[i++] = SB_LENGTH;
		else if (!strcasecmp(r, "bitrate"))
			list[i++] = SB_BITRATE;
		else if (!strcasecmp(r, "artist"))
			list[i++] = SB_ARTIST;
		else if (!strcasecmp(r, "album"))
			list[i++] = SB_ALBUM;
		else if (!strcasecmp(r, "title"))
			list[i++] = SB_TITLE;
		else if (!strcasecmp(r, "filename"))
			list[i++] = SB_FILENAME;
		else if (!strcasecmp(r, "date"))
			list[i++] = SB_DATE;
		else if (!strcasecmp(r, "filetype"))
			list[i++] = SB_FILETYPE;
		else if (!strcasecmp(r, "genre"))
			list[i++] = SB_GENRE;
	}
	if (i == ARG_NUMBER)
		i--;
	list[i] = SB_DEFAULT;
}

const char *sort_order(cmd_parms *cmd, void *d, const char *optstr)
{
	mu_config *d_cfg = (mu_config *)d;
	sort_or_fields(cmd, d_cfg->order, optstr);
	set_fctptrs(d_cfg);
	return NULL;
}

const char *set_fields(cmd_parms *cmd, void *d, const char *optstr)
{
	mu_config *d_cfg = (mu_config *)d;
	sort_or_fields(cmd, d_cfg->fields, optstr);
	return NULL;
}

/**
 * Enables the module.
 *
 * This function simply sets the module as active.
 * 
 * @param cmd I don't use it.
 * @param d Pointer to configuration structure.
 *
 * @return NULL don't ask why.
 */
const char *music_lister(cmd_parms *cmd, void *d)
{
	mu_config *d_cfg = (mu_config *)d;
	d_cfg->options |= MI_ACTIVE;
	return NULL;
}

/**
 * Enables the stream in the module.
 *
 * This function simply sets the streaming function of the module as active.
 * 
 * @param cmd I don't use it.
 * @param d Pointer to configuration structure.
 *
 * @return NULL don't ask why.
 */
const char *allow_stream(cmd_parms *cmd, void *d)
{
	mu_config *cfg = (mu_config *)d;
	cfg->options |= MI_ALLOWSTREAM;
	return NULL;
}

/**
 * Enables the download in the module.
 *
 * This function simply sets the downloading function of the module as active.
 * 
 * @param cmd I don't use it.
 * @param d Pointer to configuration structure.
 *
 * @return NULL don't ask why.
 */
const char *allow_download(cmd_parms *cmd, void *d)
{
	mu_config *cfg = (mu_config *)d;
	cfg->options |= MI_ALLOWDWNLD;
	return NULL;
}

/**
 * Enables the search in the module.
 *
 * This function simply sets the searching function of the module as active.
 * 
 * @param cmd I don't use it.
 * @param d Pointer to configuration structure.
 *
 * @return NULL don't ask why.
 */
const char *allow_search(cmd_parms *cmd, void *d)
{
	mu_config *cfg = (mu_config *)d;
	cfg->options |= MI_ALLOWSEARCH;
	return NULL;
}

/**
 * Configures the cache location in the filesystem.
 *
 * This function sets where the cache should be stored in the system.
 * 
 * @param cmd I don't use it.
 * @param d Pointer to configuration structure.
 * @param optstr The string given as parameter in the configuration.
 *
 * @return NULL don't ask why.
 */
const char *set_cache_path(cmd_parms *cmd, void *d, const char *optstr)
{
	mu_config *cfg = (mu_config *)d;
	cfg->cache_path = ap_pstrdup(cmd->pool, optstr);
	return NULL;
}

/**
 * Sets the title of the page.
 *
 * This function simply chages the name of the root.
 * 
 * @param cmd I don't use it.
 * @param d Pointer to configuration structure.
 * @param optstr The string given as parameter in the configuration.
 *
 * @return NULL don't ask why.
 */
const char *set_page_title(cmd_parms *cmd, void *d, const char *optstr)
{
	mu_config *cfg = (mu_config *)d;
	cfg->title = ap_pstrdup(cmd->pool, optstr);
	return NULL;
}

/**
 * Sets the icecast server address.
 * This function sets an icecast server for streaming.
 * 
 * @param cmd I don't use it.
 * @param d Pointer to configuration structure.
 * @param optstr The string given as parameter in the configuration.
 *
 * @return NULL don't ask why.
 */
const char *set_ice_server(cmd_parms *cmd, void *d, const char *optstr)
{
	mu_config *cfg = (mu_config *)d;
	cfg->iceserver = ap_pstrdup(cmd->pool, optstr);
	cfg->options |= MI_ALLOWSTREAM; /* Should we let the admin set it ? */
	return NULL;
}
